/*
 * OgreMax Sample Viewer and Scene Loader - Ogre3D-based viewer and code for loading and displaying .scene files
 * Copyright 2013 AND Entertainment
 *
 * This code is available under the OgreMax Free License:
 *   -You may use this code for any purpose, commercial or non-commercial.
 *   -If distributing derived works (that use this source code) in binary or source code form, 
 *    you must give the following credit in your work's end-user documentation: 
 *        "Portions of this work provided by OgreMax (www.ogremax.com)"
 *
 * AND Entertainment assumes no responsibility for any harm caused by using this code.
 * 
 * The OgreMax Sample Viewer and Scene Loader were released at www.ogremax.com 
 */


#ifndef OgreMax_Application_INCLUDED
#define OgreMax_Application_INCLUDED


//Includes---------------------------------------------------------------------
#include "ApplicationWindow.hpp"
#include "ApplicationSettings.hpp"
#include "OgrePlugins.hpp"
#include <OgreFrameListener.h>


//Classes----------------------------------------------------------------------
namespace OgreMax
{
    class ApplicationWindow;
    class IState;

    class Application : public ApplicationWindowEventListener, Ogre::FrameListener
    {
    public:
	    Application();
        ~Application();

        void SetSettings(const ApplicationSettings& settings);
        const ApplicationSettings& GetSettings() const;

        bool Run(int argc, char** argv);
        bool Run();

        static Application& GetInstance();

        /** Gets the main render window */
        Ogre::RenderWindow* GetRenderWindow() { return this->applicationWindow->GetRenderWindow(); }

        /** The main application window */
        ApplicationWindow* GetApplicationWindow() { return this->applicationWindow; }

        /** Gets the elapsed time for the last frame */
        Ogre::Real GetElapsedTime() const { return this->elapsedTime; }

        bool SuspendAllowed() const { return this->suspendAllowed; }

        void RequestToggleFullScreen() { this->toggleFullScreen = true; }

        /** Indicates whether the application window has focus */
        bool HasFocus() const { return this->applicationWindow->HasFocus(); }

        /** Indicates whether the application window has just gained focus */
        bool GainedFocus() const { return !this->hadFocus && this->applicationWindow->HasFocus(); }

        /** Indicates whether the application window has just lost focus */
        bool LostFocus() const { return this->hadFocus && !this->applicationWindow->HasFocus(); }

        /**
         * Gets the directory, if any, of the scene file specified on the command line.
         * The scene file needs to have been specified with an absolute path in order for
         * this value to not be empty
         */
        const Ogre::String& GetSceneDirectory() const { return this->sceneDirectory; }

    protected:

        /**
         * Sets up the application
         */
        virtual void Initialize();
        virtual void Shutdown();

        //Ogre::FrameListener callbacks
        bool frameEnded(const Ogre::FrameEvent& evt);

        //ApplicationWindowEventListener callbacks
        void WindowResized();
	    void WindowClosing();

    protected:
        /** The selected render system */
        Ogre::RenderSystem* renderSystem;

        /** The main render window */
        ApplicationWindow* applicationWindow;

        /** Mac OSX resource path */
        Ogre::String resourcePath;

        /** Application title and internal name */
        Ogre::String applicationTitle, applicationName;

        /** Full path to the log file */
        Ogre::String logFileName;

        /** Indicates whether the application had focus during the previous frame update */
        bool hadFocus;

        /** Indicates whether the application can be suspended */
        bool suspendAllowed;

        /** Indicates whether full screen mode should be toggled. */
        bool toggleFullScreen;

        /** Application settings */
        ApplicationSettings settings;

        /** The scene file directory. This is used when a full file path is specified on the command line */
        Ogre::String sceneDirectory;

        /** Application timing information */
        Ogre::Real elapsedTime;

        /** The current application state */
        IState* currentState;

        /** The Ogre plugins */
        OgrePlugins ogreStaticPlugins;

        static Application instance;
    };

}

#endif