/*
 * OgreMax Sample Viewer and Scene Loader - Ogre3D-based viewer and code for loading and displaying .scene files
 * Copyright 2013 AND Entertainment
 *
 * This code is available under the OgreMax Free License:
 *   -You may use this code for any purpose, commercial or non-commercial.
 *   -If distributing derived works (that use this source code) in binary or source code form, 
 *    you must give the following credit in your work's end-user documentation: 
 *        "Portions of this work provided by OgreMax (www.ogremax.com)"
 *
 * AND Entertainment assumes no responsibility for any harm caused by using this code.
 * 
 * The OgreMax Sample Viewer and Scene Loader were released at www.ogremax.com 
 */


#ifndef OgreMax_IState_INCLUDED
#define OgreMax_IState_INCLUDED


//Classes----------------------------------------------------------------------
namespace OgreMax
{
    /** A simple state machine interface */
    class IState
    {
    public:
        /**
         * Informs the state that it is becoming active
         */
        virtual void Activating() {}

        /**
         * Informs the state that it is becoming inactive
         */
        virtual void Deactivating() {}

        /**
         * Informs the state that it should perform an update
         * @return Returns a pointer to the next state or null to indicate
         * that the state machine has reached a final state.
         */
        virtual IState* Update() = 0;

        /**
         * Utility function for switching to the next state, as returned by Update()
         * @param currentState [in/out] - Reference to a pointer to the current state
         * This value is changed to reference the next state
         * @param nextState [in] - Pointer to the next state. This may be null
         */
        static void SwitchToNextState(IState*& currentState, IState* nextState)
        {
            if (nextState != currentState)
            {
                currentState->Deactivating();

                if (nextState != 0)
                    nextState->Activating();
            }
            currentState = nextState;
        }

        /**
         * Updates the current state and switches to the next state, if any
         * @param currentState [in/out] - Reference to a pointer to the current state
         * This value is changed to reference the next state
         */
        static void UpdateState(IState*& currentState)
        {
            IState* nextState = currentState->Update();
            SwitchToNextState(currentState, nextState);
        }
    };

}

#endif