/*
 * OgreMax Sample Viewer and Scene Loader - Ogre3D-based viewer and code for loading and displaying .scene files
 * Copyright 2013 AND Entertainment
 *
 * This code is available under the OgreMax Free License:
 *   -You may use this code for any purpose, commercial or non-commercial.
 *   -If distributing derived works (that use this source code) in binary or source code form, 
 *    you must give the following credit in your work's end-user documentation: 
 *        "Portions of this work provided by OgreMax (www.ogremax.com)"
 *
 * AND Entertainment assumes no responsibility for any harm caused by using this code.
 * 
 * The OgreMax Sample Viewer and Scene Loader were released at www.ogremax.com 
 */


#ifndef OgreMax_SuspendedUserActionMap_INCLUDED
#define OgreMax_SuspendedUserActionMap_INCLUDED


//Includes---------------------------------------------------------------------
#include "EventCollection.hpp"
#include "ActionMap.hpp"


//Classes----------------------------------------------------------------------
namespace OgreMax
{
    class SuspendedUserEventID
    {
    public:
        enum Type
        {
            ESCAPE,
            UNESCAPE
        };
    };

    /** User events that can occur when the program is in a 'suspended' state */
    class SuspendedUserEvents : public EventCollection<SuspendedUserEventID::Type>
    {
    public:
        SuspendedUserEvents() : EventCollection<SuspendedUserEventID::Type>()
        {
            ClearEvents();
        }

        void ClearEvents()
        {
            this->events = 0;
        }

        void AddEvent(SuspendedUserEventID::Type eventID)
        {
            if (this->canAddEvents)
                this->events |= (1 << eventID);
        }

        void RemoveEvent(SuspendedUserEventID::Type eventID)
        {
            size_t eventFlag = (1 << eventID);
            this->events &= ~eventFlag;
        }

        bool HasAnyEvent() const
        {
            return this->events != 0;
        }

        bool HasEvent(SuspendedUserEventID::Type eventID) const
        {
            size_t eventFlag = (1 << eventID);
            return (this->events & eventFlag) != 0;
        }

    public:
        /** The event flags. */
        size_t events;
    };

    class SuspendedUserActionID
    {
    public:
        enum Type
        {
            ESCAPE,
            UNESCAPE
        };
    };

    class SuspendedUserActionMap : public ActionMap<SuspendedUserActionID::Type, SuspendedUserEvents>
    {
    public:
        SuspendedUserActionMap();

    protected:
        void AddEvent
            (
            SuspendedUserEvents& events,
            SuspendedUserActionID::Type actionID,
            InputSource::DeviceType deviceType,
            TriggerFlags trigger,
            Ogre::Real amount
            );

    };

}


#endif